@@ -1,5 +1,5 @@
 import hashlib
-import importlib
+import importlib.util
 import os
 import sys
 import urllib.parse
