@@ -1,3 +1,9 @@
 from .profile_menu import ProfileMenu, select_greeter, select_profile
-from .profile_model import ProfileConfiguration
 from .profiles_handler import profile_handler
+
+__all__ = [
+    'ProfileMenu',
+    'profile_handler',
+    'select_greeter',
+    'select_profile',
+]
