@@ -7,8 +7,6 @@ from dataclasses import dataclass
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, override
 
-from .output import debug, error
-
 if TYPE_CHECKING:
 	_: Any
 
@@ -84,17 +82,6 @@ class TranslationHandler:
 
 		return languages
 
-	def _set_font(self, font: str) -> None:
-		"""
-		Set the provided font as the new terminal font
-		"""
-		from .general import SysCommand
-		try:
-			debug(f'Setting font: {font}')
-			SysCommand(f'setfont {font}')
-		except Exception:
-			error(f'Unable to set font {font}')
-
 	def _load_language_mappings(self) -> list[dict[str, str]]:
 		"""
 		Load the mapping table of all known languages
